/*============================================================================
 * General-purpose user-defined functions called before time stepping, at
 * the end of each time step, and after time-stepping.
 *
 * These can be used for operations which do not fit naturally in any other
 * dedicated user function.
 *============================================================================*/

/* VERS */

/*
  This file is part of code_saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2023 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * Local headers
 *----------------------------------------------------------------------------*/

#include "cs_headers.h"

/*----------------------------------------------------------------------------
 * Pytorch coupling headers
 *----------------------------------------------------------------------------*/

#include "annpredict.h"

#define DIM2 5
#define DIM3 8
#define EPSILON 1E-05

/*----------------------------------------------------------------------------*/

BEGIN_C_DECLS

/*----------------------------------------------------------------------------*/
/*!
 * \file cs_user_extra_operations.c
 *
 * \brief This function is called at the end of each time step, and has a very
 * general purpose (i.e. anything that does not have another dedicated
 * user function)
 */
/*----------------------------------------------------------------------------*/

/*============================================================================
 * User function definitions
 *============================================================================*/

/*----------------------------------------------------------------------------*/
/*!
 * \brief This function is called at the end of each time step.
 *
 * It has a very general purpose, although it is recommended to handle
 * mainly postprocessing or data-extraction type operations.
 *
 * \param[in, out]  domain   pointer to a cs_domain_t structure
 */
/*----------------------------------------------------------------------------*/

#pragma weak cs_user_extra_operations
void
cs_user_extra_operations(cs_domain_t     *domain)
{
  CS_UNUSED(domain);

  /* Declaration fields */
  cs_field_t *f_rho = CS_F_(rho);   /* density */
  // cs_field_t *f_vel = CS_F_(vel);   /* vel */
  cs_real_3_t *f_vel = (cs_real_3_t *)CS_F_(vel)->val;
  cs_field_t *f_temp = CS_F_(t);   /* temp */

  cs_field_t *f_f = cs_field_by_name_try("ym_fuel");   /* yfuel */
  cs_field_t *f_o = cs_field_by_name_try("ym_prod");   /* yprod */
  cs_field_t *f_p = cs_field_by_name_try("ym_oxyd");   /* yoxyd */

  const cs_lnum_t n_cells = domain->mesh->n_cells;
  const cs_time_step_t *t_step;
  cs_real_t time_cur = t_step->t_cur;

  cs_field_t *counterANN = cs_field_by_name_try("counterANN");
  int counter;
  cs_field_t *activeANN = cs_field_by_name_try("activeANN");

  /* Time values for feeding the ANN (size() = ts, n_cell, feature if present) */
  /* t0 */
  cs_field_t *rhoANN_0 = cs_field_by_name_try("rhoANN_0");
  cs_field_t *tempANN_0 = cs_field_by_name_try("tempANN_0");
  cs_real_3_t *velANN_0 = (cs_real_3_t *)cs_field_by_name_try("velANN_0")->val;
  cs_real_3_t *yiANN_0 = (cs_real_3_t *)cs_field_by_name_try("yiANN_0")->val;
  
  /* t1 */
  cs_field_t *rhoANN_1 = cs_field_by_name_try("rhoANN_1");
  cs_field_t *tempANN_1 = cs_field_by_name_try("tempANN_1");
  cs_real_3_t *velANN_1 = (cs_real_3_t *)cs_field_by_name_try("velANN_1")->val;
  cs_real_3_t *yiANN_1 = (cs_real_3_t *)cs_field_by_name_try("yiANN_1")->val;
  
  /* t2 */
  cs_field_t *rhoANN_2 = cs_field_by_name_try("rhoANN_2");
  cs_field_t *tempANN_2 = cs_field_by_name_try("tempANN_2");
  cs_real_3_t *velANN_2 = (cs_real_3_t *)cs_field_by_name_try("velANN_2")->val;
  cs_real_3_t *yiANN_2 = (cs_real_3_t *)cs_field_by_name_try("yiANN_2")->val;
  
  /* t3 */
  cs_field_t *rhoANN_3 = cs_field_by_name_try("rhoANN_3");
  cs_field_t *tempANN_3 = cs_field_by_name_try("tempANN_3");
  cs_real_3_t *velANN_3 = (cs_real_3_t *)cs_field_by_name_try("velANN_3")->val;
  cs_real_3_t *yiANN_3 = (cs_real_3_t *)cs_field_by_name_try("yiANN_3")->val;

  /* t4 */
  cs_field_t *rhoANN_4 = cs_field_by_name_try("rhoANN_4");
  cs_field_t *tempANN_4 = cs_field_by_name_try("tempANN_4");
  cs_real_3_t *velANN_4 = (cs_real_3_t *)cs_field_by_name_try("velANN_4")->val;
  cs_real_3_t *yiANN_4 = (cs_real_3_t *)cs_field_by_name_try("yiANN_4")->val;
  
  if (counterANN->val[0] < 0.1) {
    counter = 0;
  }

  /* Input to ANN */
  /* t0 */
  if (fabs(fmod(time_cur, 1E-05)) < EPSILON && counter == 0) {
    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
        rhoANN_4->val[cell_id] = f_rho->val[cell_id];
        tempANN_4->val[cell_id] = f_temp->val[cell_id];

        velANN_4[cell_id][0] = f_vel[cell_id][0];
        velANN_4[cell_id][1] = f_vel[cell_id][1];
        velANN_4[cell_id][2] = f_vel[cell_id][2];

        yiANN_4[cell_id][0] = f_f->val[cell_id];
        yiANN_4[cell_id][1] = f_o->val[cell_id];
        yiANN_4[cell_id][2] = f_p->val[cell_id];
    }
    ++counter;
  }
  /* t1 */
  else if (fabs(fmod(time_cur, 1E-05)) < EPSILON && counter == 1) {
    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
        rhoANN_3->val[cell_id] = f_rho->val[cell_id];
        tempANN_3->val[cell_id] = f_temp->val[cell_id];

        velANN_3[cell_id][0] = f_vel[cell_id][0];
        velANN_3[cell_id][1] = f_vel[cell_id][1];
        velANN_3[cell_id][2] = f_vel[cell_id][2];

        yiANN_3[cell_id][0] = f_f->val[cell_id];
        yiANN_3[cell_id][1] = f_o->val[cell_id];
        yiANN_3[cell_id][2] = f_p->val[cell_id];
    }
    ++counter;
  }
  /* t2 */
  else if (fabs(fmod(time_cur, 1E-05)) < EPSILON && counter == 2) {
    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
        rhoANN_2->val[cell_id] = f_rho->val[cell_id];
        tempANN_2->val[cell_id] = f_temp->val[cell_id];

        velANN_2[cell_id][0] = f_vel[cell_id][0];
        velANN_2[cell_id][1] = f_vel[cell_id][1];
        velANN_2[cell_id][2] = f_vel[cell_id][2];

        yiANN_2[cell_id][0] = f_f->val[cell_id];
        yiANN_2[cell_id][1] = f_o->val[cell_id];
        yiANN_2[cell_id][2] = f_p->val[cell_id];
    }
    ++counter;
  }
  /* t3 */
  else if (fabs(fmod(time_cur, 1E-05)) < EPSILON && counter == 3) {
    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
        rhoANN_1->val[cell_id] = f_rho->val[cell_id];
        tempANN_1->val[cell_id] = f_temp->val[cell_id];

        velANN_1[cell_id][0] = f_vel[cell_id][0];
        velANN_1[cell_id][1] = f_vel[cell_id][1];
        velANN_1[cell_id][2] = f_vel[cell_id][2];

        yiANN_1[cell_id][0] = f_f->val[cell_id];
        yiANN_1[cell_id][1] = f_o->val[cell_id];
        yiANN_1[cell_id][2] = f_p->val[cell_id];
    }
    ++counter;
  }
  /* t4 */
  else if (fabs(fmod(time_cur, 1E-05)) < EPSILON && counter == 4) {
    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
        rhoANN_0->val[cell_id] = f_rho->val[cell_id];
        tempANN_0->val[cell_id] = f_temp->val[cell_id];

        velANN_0[cell_id][0] = f_vel[cell_id][0];
        velANN_0[cell_id][1] = f_vel[cell_id][1];
        velANN_0[cell_id][2] = f_vel[cell_id][2];

        yiANN_0[cell_id][0] = f_f->val[cell_id];
        yiANN_0[cell_id][1] = f_o->val[cell_id];
        yiANN_0[cell_id][2] = f_p->val[cell_id];

        /* update counter */
        counterANN->val[cell_id] = 0.;
        activeANN->val[cell_id] = 1.;
    }
    counter = 0;
  }

  /*  ANN PREDICTION STEP */

  if (activeANN->val[0] > 0) {

    /* Output arrays to have overwriting */
    cs_field_t *rhoANN_o = cs_field_by_name_try("rhoANN_o");
    cs_field_t *tempANN_o = cs_field_by_name_try("tempANN_o");
    cs_real_3_t *yiANN_o = (cs_real_3_t *)cs_field_by_name_try("yiANN_o")->val;

    // allocating memory
    float ***ANNinput = (float ***)malloc(n_cells * sizeof(float **));
    for (int i = 0; i < n_cells; ++i) {
      ANNinput[i] = (float **)malloc(DIM2 * sizeof(float *));
      for (int j = 0; j < DIM2; ++j) {
        ANNinput[i][j] = (float *)malloc(DIM3 * sizeof(float));
      }
    }
    // assigning values -> v_x v_y v_z rho T y_fuel y_oxyd y_prod
    for (int i = 0; i < n_cells; ++i) {
      /* t0 */
      ANNinput[i][0][0] = velANN_4[i][0];
      ANNinput[i][0][1] = velANN_4[i][1];
      ANNinput[i][0][2] = velANN_4[i][2];
      ANNinput[i][0][3] = rhoANN_4->val[i];
      ANNinput[i][0][4] = tempANN_4->val[i];
      ANNinput[i][0][5] = yiANN_4[i][0];
      ANNinput[i][0][6] = yiANN_4[i][1];
      ANNinput[i][0][7] = yiANN_4[i][2];
      /* t1 */
      ANNinput[i][1][0] = velANN_3[i][0];
      ANNinput[i][1][1] = velANN_3[i][1];
      ANNinput[i][1][2] = velANN_3[i][2];
      ANNinput[i][1][3] = rhoANN_3->val[i];
      ANNinput[i][1][4] = tempANN_3->val[i];
      ANNinput[i][1][5] = yiANN_3[i][0];
      ANNinput[i][1][6] = yiANN_3[i][1];
      ANNinput[i][1][7] = yiANN_3[i][2];
      /* t2 */
      ANNinput[i][2][0] = velANN_2[i][0];
      ANNinput[i][2][1] = velANN_2[i][1];
      ANNinput[i][2][2] = velANN_2[i][2];
      ANNinput[i][2][3] = rhoANN_2->val[i];
      ANNinput[i][2][4] = tempANN_2->val[i];
      ANNinput[i][2][5] = yiANN_2[i][0];
      ANNinput[i][2][6] = yiANN_2[i][1];
      ANNinput[i][2][7] = yiANN_2[i][2];
      /* t3 */
      ANNinput[i][3][0] = velANN_1[i][0];
      ANNinput[i][3][1] = velANN_1[i][1];
      ANNinput[i][3][2] = velANN_1[i][2];
      ANNinput[i][3][3] = rhoANN_1->val[i];
      ANNinput[i][3][4] = tempANN_1->val[i];
      ANNinput[i][3][5] = yiANN_1[i][0];
      ANNinput[i][3][6] = yiANN_1[i][1];
      ANNinput[i][3][7] = yiANN_1[i][2];
      /* t4 */
      ANNinput[i][4][0] = velANN_0[i][0];
      ANNinput[i][4][1] = velANN_0[i][1];
      ANNinput[i][4][2] = velANN_0[i][2];
      ANNinput[i][4][3] = rhoANN_0->val[i];
      ANNinput[i][4][4] = tempANN_0->val[i];
      ANNinput[i][4][5] = yiANN_0[i][0];
      ANNinput[i][4][6] = yiANN_0[i][1];
      ANNinput[i][4][7] = yiANN_0[i][2];

    }

    float** ANNoutput = ANNpredict(ANNinput);   // forward through model

    for (int i = 0; i < n_cells; ++i) {
      rhoANN_o->val[i] = ANNoutput[i][3];
      tempANN_o->val[i] = ANNoutput[i][4];
      yiANN_o[i][0] = ANNoutput[i][5];
      yiANN_o[i][1] = ANNoutput[i][6];
      yiANN_o[i][2] = ANNoutput[i][7];
    }
    /* update time iteration (dtANN = 10E-05) */
    cs_time_step_redefine_cur(t_step->nt_cur,
                              time_cur+(1E-05)-(t_step->dt_ref));
  }
}

/*----------------------------------------------------------------------------*/

END_C_DECLS
