/*============================================================================
 * User functions for input of calculation parameters.
 *============================================================================*/

/* VERS */

/*
  This file is part of code_saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2023 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>
#include <string.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 * Local headers
 *----------------------------------------------------------------------------*/

#include "cs_headers.h"

/*----------------------------------------------------------------------------*/

BEGIN_C_DECLS

/*----------------------------------------------------------------------------*/
/*!
 * \file cs_user_parameters-base.c
 *
 * \brief User functions for input of calculation parameters.
 *
 * See \ref parameters for examples.
 */
/*----------------------------------------------------------------------------*/

/*============================================================================
 * User function definitions
 *============================================================================*/

/*----------------------------------------------------------------------------*/
/*!
 * \brief Select physical model options, including user fields.
 *
 * This function is called at the earliest stages of the data setup,
 * so field ids are not available yet.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_model(void)
{
  /* Pinzo@10092024: parameter to check the ANN activation in each cell */ 
  cs_parameters_add_property("activeANN",
                             1,
                             CS_MESH_LOCATION_CELLS);
  
  /* Pinzo@12092024: Adding ANN input variables */
  /* t0 */
  /* density */
  cs_parameters_add_variable("rhoANN_0", 1);

  /* temp */
  cs_parameters_add_variable("tempANN_0", 1);
                     
  /* vel */
  cs_parameters_add_variable("velANN_0", 3);

  /* yi */
  cs_parameters_add_variable("yiANN_0", 3);

  /* t1 */
  /* density */
  cs_parameters_add_variable("rhoANN_1", 1);

  /* temp */
  cs_parameters_add_variable("tempANN_1", 1);
                     
  /* vel */
  cs_parameters_add_variable("velANN_1", 3);

  /* yi */
  cs_parameters_add_variable("yiANN_1", 3);

  /* t2 */
  /* density */
  cs_parameters_add_variable("rhoANN_2", 1);

  /* temp */
  cs_parameters_add_variable("tempANN_2", 1);
                     
  /* vel */
  cs_parameters_add_variable("velANN_2", 3);

  /* yi */
  cs_parameters_add_variable("yiANN_2", 3);

  /* t3 */
  /* density */
  cs_parameters_add_variable("rhoANN_3", 1);

  /* temp */
  cs_parameters_add_variable("tempANN_3", 1);
                     
  /* vel */
  cs_parameters_add_variable("velANN_3", 3);

  /* yi */
  cs_parameters_add_variable("yiANN_3", 3);

  /* t4 */
  /* density */
  cs_parameters_add_variable("rhoANN_4", 1);

  /* temp */
  cs_parameters_add_variable("tempANN_4", 1);
                     
  /* vel */
  cs_parameters_add_variable("velANN_4", 3);

  /* yi */
  cs_parameters_add_variable("yiANN_4", 3);
  /* (END)Pinzo@12092024: Adding ANN input variables */

  /* Pinzo@12092024: Adding ANN output variables */
  /* density */
  cs_parameters_add_variable("rhoANN_o", 1);

  /* temp */
  cs_parameters_add_variable("tempANN_o", 1);
                     
  /* vel */
  cs_parameters_add_variable("velANN_o", 3);

  /* yi */
  cs_parameters_add_variable("yiANN_o", 3);

  /* Pinzo@16092024: Adding counter varaible for collection purposes */
  cs_parameters_add_variable("counterANN", 1);

}

/* Pinzo@10092024: Fixing the correct numerical parameters in LES combustion */
void
cs_user_parameters(cs_domain_t *domain)
{
    int n_fields = cs_field_n_fields();

    for (int f_id = 0; f_id < n_fields; f_id++) {

      cs_field_t *f = cs_field_by_id(f_id);

      if (f->type & CS_FIELD_VARIABLE) {
        cs_equation_param_t *eqp ;   // = cs_field_get_equation_param(f);
        eqp = cs_field_get_equation_param(f);
        eqp->blencv = 0.;   // upwind scheme: blencv = 0;
      }
    }
}
END_C_DECLS
