!-------------------------------------------------------------------------------

!VERS

! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2021 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!============================================!
!== Pinzo@11092024: Camille's code ==
subroutine Zv_extra_op()
    use paramx
    use pointe
    use numvar
    use optcal
    use cstphy
    use cstnum
    use entsor
    use parall
    use period
    use ppppar
    use ppthch
    use ppincl
    use field
    use mesh
    use lagran
    use cs_c_bindings
    use coincl
    use radiat
    use dimens, only: ndimfb
    use field_operator

    implicit none
    integer iprev, iel
    integer inc, iccocg

    double precision, allocatable, dimension(:,:) :: grad
    double precision, dimension(:), pointer :: cvar_fp2m
    double precision :: grad_mag, grad_sqr, deltaK, fp2m

    allocate(grad(3, ncelet))
    call field_get_val_s(ivarfl(isca(ifp2m)), cvar_fp2m)   ! get field values of the scalar cvar_f2pm; isca(ifp2m): index of scalar mixfrac variance; ivarfl(): field id

    ! calculate mixture fraction variance
    iprev = 0        ! 1 = value at previous iteration, 0 = otherwise
    inc = 1          ! 1 = do not increment, 0 = otherwise
    iccocg = 1       ! 1 = recompute COCG, 0 = otherwise
    deltaK = 1E-03  

    call field_gradient_scalar(ivarfl(isca(ifp2m)), iprev, imrgra, inc,  &
                               iccocg,                                   &
                               grad)
    
    ! Zv calculation
    do iel = 1, ncel
        grad_mag = sqrt(grad(1,iel)*grad(1,iel)+                           &
                        grad(2,iel)*grad(2,iel)+                           & 
                        grad(3,iel)*grad(3,iel))   ! variance magnitude
        
        grad_sqr = grad_mag*grad_mag

        ! variance Z calculation 
        fp2m = 0.18*(volume(iel)**((2.d0)/(3.d0)))*grad_sqr

        ! secure value by clipping it between 0 and Z(1-Z)
        if (fp2m.lt.0.0) then
            cvar_fp2m(iel) = 0.d0
        else
            cvar_fp2m(iel) = fp2m
        endif
    enddo

    deallocate(grad)   ! release memory

end subroutine Zv_extra_op

!== (END)Pinzo@11092024: Camille's code ==
!============================================!

!===============================================================================
!> \brief Modify turbulent viscosity
!>
!> This subroutine is called at beginning of each time step
!> after the computation of the turbulent viscosity
!> (physical quantities have already been computed in \ref usphyv).
!>
!> Turbulent viscosity \f$ \mu_T \f$ (kg/(m s)) can be modified.
!>
!> A modification of the turbulent viscosity can lead to very
!> significant differences between solutions and even give wrong
!> results.
!>
!> This subroutine is therefore reserved to expert users.
!
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     ncepdp        number of cells with head loss
!> \param[in]     ncesmp        number of cells with mass source term
!> \param[in]     icepdc        head loss cell numbering
!> \param[in]     icetsm        numbering of cells with mass source term
!> \param[in]     itypsm        kind of mass source for each variable
!> \param[in]     dt            time step (per cell)
!> \param[in]     ckupdc        work array for head loss terms
!> \param[in]     smacel        values of variables related to mass source
!>                              term. If ivar=ipr, smacel=mass flux
!_______________________________________________________________________________

subroutine usvist &
    ( nvar   , nscal  , ncepdp , ncesmp ,                            &
      icepdc , icetsm , itypsm ,                                     &
      dt     ,                                                       &
      ckupdc , smacel )
   
   !===============================================================================
   
   !===============================================================================
   ! Module files
   !===============================================================================
   
   use paramx
   use numvar
   use optcal
   use cstphy
   use entsor
   use parall
   use period
   use mesh
   use field
   use field_operator
   
   !===============================================================================
   
   implicit none
   
   ! Arguments
   
   integer          nvar   , nscal
   integer          ncepdp , ncesmp
   
   integer          icepdc(ncepdp)
   integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
   
   double precision dt(ncelet)
   double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
   
   !===============================================================================
   
   !--------
   ! Formats
   !--------
   
   !----
   ! End
   !----
   
   return
   end subroutine usvist
   
   !===============================================================================
   
   !===============================================================================
   !> \brief user modification of the Smagorinsky constant in the case of a
   !>   dynamic model
   !>
   !>              SMAGOR = Mij.Lij / Mij.Mij
   !>
   !> The local averages of the numerator and denominator are done before calling
   !> this subroutine, so
   !>
   !>              SMAGOR = < Mij.Lij > / < Mij.Mij >
   !>
   !> In this subroutine, Mij.Lij and Mij.Mij are passed as arguments before
   !> the local average.
   
   !-------------------------------------------------------------------------------
   ! Arguments
   !______________________________________________________________________________.
   !  mode           name          role                                           !
   !______________________________________________________________________________!
   !> \param[in]     nvar          total number of variables
   !> \param[in]     nscal         total number of scalars
   !> \param[in]     ncepdp        number of cells with head loss
   !> \param[in]     ncesmp        number of cells with mass source term
   !> \param[in]     icepdc        head loss cell numbering
   !> \param[in]     icetsm        numbering of cells with mass source term
   !> \param[in]     itypsm        kind of mass source for each variable
   !> \param[in]     dt            time step (per cell)
   !> \param[in]     ckupdc        work array for head loss terms
   !> \param[in]     smacel        values of variables related to mass source
   !>                              term. If ivar=ipr, smacel=mass flux
   !> \param[in]     mijlij        mij.lij before the local averaging
   !> \param[in]     mijmij        mij.mij before the local averaging
   !_______________________________________________________________________________
   
   subroutine ussmag &
    ( nvar   , nscal  , ncepdp , ncesmp ,                            &
      icepdc , icetsm , itypsm ,                                     &
      dt     ,                                                       &
      ckupdc , smacel ,                                              &
      mijlij , mijmij )
   
   !===============================================================================
   
   !===============================================================================
   ! Module files
   !===============================================================================
   
   use paramx
   use numvar
   use cstnum
   use optcal
   use cstphy
   use entsor
   use parall
   use field
   use mesh
   
   !===============================================================================
   
   implicit none
   
   ! Arguments
   
   integer          nvar   , nscal
   integer          ncepdp , ncesmp
   
   integer          icepdc(ncepdp)
   integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
   
   double precision dt(ncelet)
   double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
   double precision mijlij(ncelet), mijmij(ncelet)
   
   ! Local
   
   double precision, dimension(:), pointer :: cpro_smago
   
   call field_get_val_s(ismago, cpro_smago)
   
   !===============================================================================
   
   !----
   ! End
   !----
   
   return
   end subroutine ussmag
   
   !===============================================================================